@@ -39,7 +39,7 @@ constparties <- function(nodes, data, weights, fitted = NULL, terms = NULL, info
             rw <- rmultinom(1, size = length(prob), prob = prob)
         } else {
             rw <- integer(length(prob))
-            i <- sample(1:length(prob), floor(fraction * length(prob)), prob = prob)
+            i <- sample(1:length(prob), ceiling(fraction * length(prob)), prob = prob)
             rw[i] <- 1L
         }
         as.integer(rw)
@@ -47,111 +47,119 @@ constparties <- function(nodes, data, weights, fitted = NULL, terms = NULL, info
     ret
 }
 
-cforest <- function(formula, data, weights, subset, na.action = na.pass, 
-                    control = ctree_control(teststat = "quad",
-                                            testtype = "Univ", mincriterion = 0, ...), 
-                    ytrafo = NULL, scores = NULL,
-                    ntree = 500L, perturb = list(replace = FALSE, fraction = 0.632),
-                    mtry = ceiling(sqrt(nvar)), applyfun = NULL, cores = NULL, 
-                    trace = FALSE, ...) {
+cforest <- function
+(
+    formula,
+    data,   
+    weights,
+    subset, 
+    offset, 
+    cluster,
+    strata,
+    na.action = na.pass,
+    control = ctree_control(
+        teststat = "quad", testtype = "Univ", mincriterion = 0,
+        saveinfo = FALSE, ...),
+    ytrafo = NULL, 
+    scores = NULL, 
+    ntree = 500L, 
+    perturb = list(replace = FALSE, fraction = 0.632),
+    mtry = ceiling(sqrt(nvar)), 
+    applyfun = NULL,
+    cores = NULL, 
+    trace = FALSE,
+    ...
+) {
+   
+    ### get the call and the calling environment for .urp_tree
+    call <- match.call(expand.dots = FALSE)
+    m <- match(c("formula", "data", "subset", "na.action", "weights", "offset", "cluster", 
+                 "scores", "ytrafo", "control"), names(call), 0L)
+    ctreecall <- call[c(1L, m)]
+    ctreecall$doFit <- FALSE
+    ctreecall$control <- control ### put ... into ctree_control()
+    ctreecall[[1L]] <- quote(partykit::ctree)
+    tree <- eval(ctreecall, parent.frame())
 
-    if (missing(data))
-        data <- environment(formula)
-    mf <- match.call(expand.dots = FALSE)
-    m <- match(c("formula", "data", "subset", "weights", "na.action"),
-               names(mf), 0)
-    mf <- mf[c(1, m)]
-    
-    ### only necessary for extended model formulae 
-    ### e.g. multivariate responses
-    formula <- Formula::Formula(formula)
-    mf$formula <- formula
-    mf$drop.unused.levels <- FALSE
-    mf$na.action <- na.action
-    mf[[1]] <- quote(stats::model.frame)
-    mf <- eval(mf, parent.frame())
+    if (is.null(control$update))
+        control$update <- is.function(ytrafo)
 
-    response <- names(Formula::model.part(formula, mf, lhs = 1))
-    weights <- model.weights(mf)
-    dat <- mf[, colnames(mf) != "(weights)"]
-    if (!is.null(scores)) {
-        for (n in names(scores)) {
-            sc <- scores[[n]]
-            if (is.ordered(dat[[n]]) && 
-                nlevels(dat[[n]]) == length(sc)) {
-                attr(dat[[n]], "scores") <- as.numeric(sc)
-            } else {
-                warning("scores for variable ", sQuote(n), " ignored")
-            }
-        }
-    }
+    d <- tree$d
+    updatefun <- tree$update
 
-    if (is.null(weights))
-        weights <- rep(1, nrow(mf))
-
-    nvar <- sum(!(colnames(dat) %in% response)) 
+    nvar <- sum(d$variables$z > 0)
     control$mtry <- mtry
-
-    control$cfun <- function(...) {
-        if (control$teststat == "quad")
-            p <- .pX2(..., pval = (control$testtype != "Teststatistic"))
-        if (control$teststat == "max") 
-            p <- .pmaxT(..., pval = (control$testtype != "Teststatistic"))
-        names(p) <- c("statistic", "p.value")
-
-        if (control$testtype == "Bonferroni")
-            p["p.value"] <- p["p.value"] * min(nvar, control$mtry)
-        crit <-  p["statistic"]
-        if (control$testtype != "Teststatistic")
-        crit <- p["p.value"]
-        c(crit, p)
+    control$applyfun <- lapply
+ 
+    strata <- d[["(strata)"]]
+    if (!is.null(strata)) {
+        if (!is.factor(strata)) stop("strata is not a single factor")
+    }
+    
+    probw <- NULL
+    weights <- model.weights(model.frame(d))
+    if (!is.null(weights)) {
+        probw <- weights / sum(weights)
+    } else {
+        weights <- integer(0)
     }
 
-    perturb <- do.call(".perturb", perturb)
-
-    if (!is.matrix(weights)) {
-        probw <- weights / sum(weights)
-        rw <- replicate(ntree, perturb(probw), simplify = FALSE)
+    N <- nrow(model.frame(d))
+    idx <- .start_subset(d)
+    if (is.null(strata)) {
+        size <- N
+        if (!perturb$replace) size <- floor(size * perturb$fraction)
+        rw <- replicate(ntree, sample(idx, size = size, replace = perturb$replace, prob = probw),
+                        simplify = FALSE)
     } else {
-        stopifnot(nrow(weights) == nrow(dat) && ncol(weights) == ntree)
-        rw <- as.data.frame(weights)
-        class(rw) <- "list"
+        frac <- if (!perturb$replace) perturb$fraction else 1
+        rw <- replicate(ntree, function() 
+            do.call("c", tapply(idx, strata, function(i) sample(i, size = length(i) * frac, 
+                   replace = perturb$replace, prob = probw[i]))))
     }
 
     ## apply infrastructure for determining split points
     if (is.null(applyfun)) {
         applyfun <- if(is.null(cores)) {
-            lapply
+            lapply  
         } else {
-            function(X, FUN, ...) 
+            function(X, FUN, ...)
                 parallel::mclapply(X, FUN, ..., mc.cores = cores)
         }
     }
-    ### no parallelization in ctree, only in cforest
-    control$applyfun <- lapply
 
-    if (trace) pb <- txtProgressBar(style = 3)
+    trafo <- updatefun(sort(rw[[1]]), integer(0), control, doFit = FALSE)
+    if (trace) pb <- txtProgressBar(style = 3) 
     forest <- applyfun(1:ntree, function(b) {
         if (trace) setTxtProgressBar(pb, b/ntree)
-        .ctree_fit(dat, response, weights = rw[[b]], 
-                   ctrl = control, ytrafo = ytrafo)
+        ret <- updatefun(sort(rw[[b]]), integer(0), control)
+        trafo <<- ret$trafo
+        ret$nodes
     })
     if (trace) close(pb)
 
-    fitted <- data.frame(idx = 1:nrow(dat))
-    fitted[[2]] <- dat[,response, drop = length(response) == 1]
+    fitted <- data.frame(idx = 1:N)  
+    mf <- model.frame(d)
+    fitted[[2]] <- mf[, d$variables$y, drop = TRUE]
     names(fitted)[2] <- "(response)"
-    fitted <- fitted[2]
+    if (length(weights) > 0)
+        fitted[["(weights)"]] <- weights
 
-    ret <- constparties(nodes = forest, data = dat, weights = rw,
-                        fitted = fitted, terms = terms(mf), 
+    ### turn subsets in weights (maybe we can avoid this?)
+    rw <- lapply(rw, function(x) as.integer(tabulate(x, nbins = length(idx))))
+
+    control$applyfun <- applyfun
+
+    ret <- constparties(nodes = forest, data = mf, weights = rw,
+                        fitted = fitted, terms = d$terms$all,
                         info = list(call = match.call(), control = control))
+    ret$trafo <- trafo
+    ret$predictf <- d$terms$z
     class(ret) <- c("cforest", class(ret))
 
     return(ret)
 }
 
-
 predict.cforest <- function(object, newdata = NULL, type = c("response", "prob", "weights", "node"), 
                             OOB = FALSE, FUN = NULL, simplify = TRUE, ...) {
 
@@ -159,11 +167,13 @@ predict.cforest <- function(object, newdata = NULL, type = c("response", "prob",
     forest <- object$nodes
     nd <- object$data
     vmatch <- 1:ncol(nd)
+    NOnewdata <- TRUE
     if (!is.null(newdata)) {
-        nd <- model.frame(delete.response(object$terms), 
+        nd <- model.frame(object$predictf, ### all variables W/O response
                           data = newdata, na.action = na.pass)
         OOB <- FALSE
         vmatch <- match(names(object$data), names(nd))
+        NOnewdata <- FALSE
     }
     nam <- rownames(nd)
 
@@ -176,24 +186,35 @@ predict.cforest <- function(object, newdata = NULL, type = c("response", "prob",
     ### extract weights
     rw <- object$weights
 
-    # w <- matrix(0L, nrow = NROW(responses), ncol = length(nam))
+    w <- 0L
 
     applyfun <- lapply
     if (!is.null(object$info))
         applyfun <- object$info$control$applyfun
 
-    bw <- applyfun(1:length(forest), function(b) {
-        ids <- nodeids(forest[[b]], terminal = TRUE)
-        fnewdata <- fitted_node(forest[[b]], nd, vmatch = vmatch, ...)
-        fdata <- fitted_node(forest[[b]], object$data, ...)
-        tw <- rw[[b]]
-        if (OOB) tw <- as.integer(tw == 0)
-        pw <- sapply(ids, function(i) tw * (fdata == i))
-        return(pw[, match(fnewdata, ids), drop = FALSE])
-    })
+    fdata <- lapply(forest, fitted_node, data = object$data, ...)
+    if (NOnewdata && OOB) {
+        fnewdata <- list()
+    } else {
+        fnewdata <- lapply(forest, fitted_node, data = nd, vmatch = vmatch, ...)
+    }
 
-    w <- Reduce("+", bw)
-    if (!is.matrix(w)) w <- matrix(w, ncol = 1)
+    w <- .rfweights(fdata, fnewdata, rw)
+
+#    for (b in 1:length(forest)) {
+#        ids <- nodeids(forest[[b]], terminal = TRUE)
+#        fnewdata <- fitted_node(forest[[b]], nd, vmatch = vmatch, ...)
+#        fdata <- fitted_node(forest[[b]], object$data, ...)
+#        tw <- rw[[b]]
+#        pw <- sapply(ids, function(i) tw * (fdata == i))
+#        ret <- pw[, match(fnewdata, ids), drop = FALSE]
+#        ### obs which are in-bag for this tree don't contribute
+#        if (OOB) ret[,tw > 0] <- 0
+#        w <- w + ret
+#    }
+#
+#    #w <- Reduce("+", bw)
+#    if (!is.matrix(w)) w <- matrix(w, ncol = 1)
 
     if (type == "weights") {
         ret <- w
@@ -237,3 +258,8 @@ predict.cforest <- function(object, newdata = NULL, type = c("response", "prob",
     }
     ret
 }
+
+model.frame.cforest <- function(formula, ...) {
+    class(formula) <- "party"
+    model.frame(formula, ...)
+}
