@@ -35,7 +35,7 @@ glmtree <- function(formula, data, subset, na.action, weights, offset, cluster,
   m$maxit <- maxit
   if("..." %in% names(m)) m[["..."]] <- NULL
   if(!fam) m$family <- family
-  m[[1L]] <- as.name("mob")
+  m[[1L]] <- as.call(quote(partykit::mob))
   rval <- eval(m, parent.frame())
   
   ## extend class and keep original call
@@ -90,7 +90,7 @@ glmfit <- function(y, x, start = NULL, weights = NULL, offset = NULL, cluster =
     } else {
       sum(wres^2, na.rm = TRUE)/sum(z$weights, na.rm = TRUE)
     }
-    rval$estfun <- wres * x/dispersion
+    rval$estfun <- wres * x[, !is.na(z$coefficients), drop = FALSE]/dispersion
   }
 
   ## add model (if desired)
